@@ -12,7 +12,8 @@
 
 set -e
 
-SCRIPTS_DIR=$(cd "$(dirname "$0")" || exit 1; pwd)
+SCRIPT=$(readlink -f "${BASH_SOURCE[0]}")
+SCRIPTS_DIR=$(dirname ${SCRIPT})
 BASE_DIR="$1"
 QUIET=""
 
@@ -29,7 +30,7 @@ command -v skopeo > /dev/null 2>&1 || { echo "skopeo is not installed. Aborting.
 
 usage () {
 	echo "Usage:   $0 [-w WORKDIR] -c [/path/to/csv.yaml] -t [IMAGE_TAG]"
-	echo "Example: $0 -w $(pwd) -c $(pwd)/deploy/olm-catalog/nightly/eclipse-che-preview-kubernetes/manifests/che-operator.clusterserviceversion.yaml -t 7.26.0"
+	echo "Example: $0 -w $(pwd) -c $(pwd)/bundle/nightly/eclipse-che-preview-kubernetes/manifests/che-operator.clusterserviceversion.yaml -t 7.26.0"
 }
 
 if [[ $# -lt 1 ]]; then usage; exit; fi
@@ -60,6 +61,9 @@ setImagesFromDeploymentEnv
 setOperatorImage
 echo "${OPERATOR_IMAGE}"
 
+setDevWorkspaceCheOperatorImage
+echo ${DEVWORKSPACE_CHE_OPERATOR_IMAGE}
+
 setPluginRegistryList
 echo "${PLUGIN_REGISTRY_LIST}"
 
@@ -101,7 +105,7 @@ writeDigest() {
       digest="$(skopeo inspect --tls-verify=false docker://${image} 2>/dev/null | jq -r '.Digest')"
       fi
       if [[ -z ${digest} ]]; then
-        echo "==================== Failed to get digest for image: ${image}======================"
+        echo "[ERROR] Failed to get digest for image: ${image}"
         withoutTag=""
         withDigest=""
       else
@@ -126,9 +130,10 @@ rm -Rf "${DIGEST_FILE}"
 touch "${DIGEST_FILE}"
 
 writeDigest "${OPERATOR_IMAGE}" "operator-image"
+writeDigest "${DEVWORKSPACE_CHE_OPERATOR_IMAGE}" "devworkspace-che-operator-image"
 
 for image in ${REQUIRED_IMAGES}; do
-  writeDigest "${image}" "required-image" 
+  writeDigest "${image}" "required-image"
 done
 
 for image in ${PLUGIN_REGISTRY_LIST}; do
