@@ -17,11 +17,13 @@ import (
 	"errors"
 	"fmt"
 	"io/ioutil"
+	"strings"
 
-	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	orgv1 "github.com/eclipse-che/che-operator/api/v1"
 	"github.com/eclipse-che/che-operator/pkg/deploy"
 	"github.com/eclipse-che/che-operator/pkg/util"
 	operatorsv1alpha1 "github.com/operator-framework/api/pkg/operators/v1alpha1"
+	"github.com/sirupsen/logrus"
 	admissionregistrationv1 "k8s.io/api/admissionregistration/v1"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -31,7 +33,6 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
-	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/yaml"
@@ -39,20 +40,20 @@ import (
 
 var (
 	DevWorkspaceNamespace      = "devworkspace-controller"
-	DevWorkspaceCheNamespace   = "devworkspace-che"
 	DevWorkspaceWebhookName    = "controller.devfile.io"
 	DevWorkspaceServiceAccount = "devworkspace-controller-serviceaccount"
+	DevWorkspaceService        = "devworkspace-controller-manager-service"
 	DevWorkspaceDeploymentName = "devworkspace-controller-manager"
-	SubscriptionResourceName   = "subscriptions"
-	CheManagerResourcename     = "chemanagers"
 
-	OpenshiftDevWorkspaceTemplatesPath     = "/tmp/devworkspace-operator/templates/deployment/openshift/objects"
-	OpenshiftDevWorkspaceCheTemplatesPath  = "/tmp/devworkspace-che-operator/templates/deployment/openshift/objects/"
-	KubernetesDevWorkspaceTemplatesPath    = "/tmp/devworkspace-operator/templates/deployment/kubernetes/objects"
-	KubernetesDevWorkspaceCheTemplatesPath = "/tmp/devworkspace-che-operator/templates/deployment/kubernetes/objects/"
+	SubscriptionResourceName          = "subscriptions"
+	CheManagerResourcename            = "chemanagers"
+	ClusterServiceVersionResourceName = "clusterserviceversions"
+	DevWorkspaceCSVNameWithouVersion  = "devworkspace-operator"
 
-	DevWorkspaceTemplates    = devWorkspaceTemplatesPath()
-	DevWorkspaceCheTemplates = devWorkspaceCheTemplatesPath()
+	OpenshiftDevWorkspaceTemplatesPath  = "/tmp/devworkspace-operator/templates/deployment/openshift/objects"
+	KubernetesDevWorkspaceTemplatesPath = "/tmp/devworkspace-operator/templates/deployment/kubernetes/objects"
+
+	DevWorkspaceTemplates = devWorkspaceTemplatesPath()
 
 	DevWorkspaceServiceAccountFile            = DevWorkspaceTemplates + "/devworkspace-controller-serviceaccount.ServiceAccount.yaml"
 	DevWorkspaceRoleFile                      = DevWorkspaceTemplates + "/devworkspace-controller-leader-election-role.Role.yaml"
@@ -67,20 +68,10 @@ var (
 	DevWorkspaceTemplatesCRDFile              = DevWorkspaceTemplates + "/devworkspacetemplates.workspace.devfile.io.CustomResourceDefinition.yaml"
 	DevWorkspaceCRDFile                       = DevWorkspaceTemplates + "/devworkspaces.workspace.devfile.io.CustomResourceDefinition.yaml"
 	DevWorkspaceConfigMapFile                 = DevWorkspaceTemplates + "/devworkspace-controller-configmap.ConfigMap.yaml"
+	DevWorkspaceServiceFile                   = DevWorkspaceTemplates + "/devworkspace-controller-manager-service.Service.yaml"
 	DevWorkspaceDeploymentFile                = DevWorkspaceTemplates + "/devworkspace-controller-manager.Deployment.yaml"
-
-	DevWorkspaceCheServiceAccountFile           = DevWorkspaceCheTemplates + "/devworkspace-che-serviceaccount.ServiceAccount.yaml"
-	DevWorkspaceCheRoleFile                     = DevWorkspaceCheTemplates + "/devworkspace-che-leader-election-role.Role.yaml"
-	DevWorkspaceCheClusterRoleFile              = DevWorkspaceCheTemplates + "/devworkspace-che-role.ClusterRole.yaml"
-	DevWorkspaceCheProxyClusterRoleFile         = DevWorkspaceCheTemplates + "/devworkspace-che-proxy-role.ClusterRole.yaml"
-	DevWorkspaceCheMetricsReaderClusterRoleFile = DevWorkspaceCheTemplates + "/devworkspace-che-metrics-reader.ClusterRole.yaml"
-	DevWorkspaceCheRoleBindingFile              = DevWorkspaceCheTemplates + "/devworkspace-che-leader-election-rolebinding.RoleBinding.yaml"
-	DevWorkspaceCheClusterRoleBindingFile       = DevWorkspaceCheTemplates + "/devworkspace-che-rolebinding.ClusterRoleBinding.yaml"
-	DevWorkspaceCheProxyClusterRoleBindingFile  = DevWorkspaceCheTemplates + "/devworkspace-che-proxy-rolebinding.ClusterRoleBinding.yaml"
-	DevWorkspaceCheManagersCRDFile              = DevWorkspaceCheTemplates + "/chemanagers.che.eclipse.org.CustomResourceDefinition.yaml"
-	DevWorkspaceCheConfigMapFile                = DevWorkspaceCheTemplates + "/devworkspace-che-configmap.ConfigMap.yaml"
-	DevWorkspaceCheDeploymentFile               = DevWorkspaceCheTemplates + "/devworkspace-che-manager.Deployment.yaml"
-	DevWorkspaceCheMetricsServiceFile           = DevWorkspaceCheTemplates + "/devworkspace-che-controller-manager-metrics-service.Service.yaml"
+	DevWorkspaceIssuerFile                    = DevWorkspaceTemplates + "/devworkspace-controller-selfsigned-issuer.Issuer.yaml"
+	DevWorkspaceCertificateFile               = DevWorkspaceTemplates + "/devworkspace-controller-serving-cert.Certificate.yaml"
 
 	WebTerminalOperatorSubscriptionName = "web-terminal"
 	WebTerminalOperatorNamespace        = "openshift-operators"
@@ -96,6 +87,7 @@ var (
 	cachedObj = make(map[string]*Object2Sync)
 	syncItems = []func(*deploy.DeployContext) (bool, error){
 		createDwNamespace,
+		syncDwService,
 		syncDwServiceAccount,
 		syncDwClusterRole,
 		syncDwProxyClusterRole,
@@ -105,36 +97,39 @@ var (
 		syncDwRoleBinding,
 		syncDwClusterRoleBinding,
 		syncDwProxyClusterRoleBinding,
+		syncDwIssuer,
+		syncDwCertificate,
 		syncDwCRD,
 		syncDwTemplatesCRD,
 		syncDwWorkspaceRoutingCRD,
 		syncDwConfigMap,
 		syncDwDeployment,
 	}
-
-	syncDwCheItems = []func(*deploy.DeployContext) (bool, error){
-		createDwCheNamespace,
-		syncDwCheServiceAccount,
-		syncDwCheClusterRole,
-		syncDwCheProxyClusterRole,
-		syncDwCheMetricsClusterRole,
-		syncDwCheLeaderRole,
-		syncDwCheLeaderRoleBinding,
-		syncDwCheProxyRoleBinding,
-		syncDwCheRoleBinding,
-		syncDwCheCRD,
-		synDwCheCR,
-		syncDwCheConfigMap,
-		syncDwCheMetricsService,
-		synDwCheDeployment,
-	}
 )
 
 func ReconcileDevWorkspace(deployContext *deploy.DeployContext) (bool, error) {
+	if util.IsOpenShift && !util.IsOpenShift4 {
+		// OpenShift 3.x is not supported
+		return true, nil
+	}
+
+	// Check if exists devworkspace operator csv is already installed
+	devWorkspaceOperatorCSVExists := isDevWorkspaceOperatorCSVExists(deployContext)
+	if devWorkspaceOperatorCSVExists {
+		return true, nil
+	}
+
+	// do nothing if dev workspace is disabled
 	if !deployContext.CheCluster.Spec.DevWorkspace.Enable {
 		return true, nil
 	}
 
+	if !util.IsOpenShift && util.GetCheServerCustomCheProperty(deployContext.CheCluster, "CHE_INFRA_KUBERNETES_ENABLE__UNSUPPORTED__K8S") != "true" {
+		logrus.Warn(`DevWorkspace Che operator can't be enabled on a Kubernetes cluster without explicitly enabled k8s API on che-server. To enable DevWorkspace Che operator set 'spec.server.customCheProperties[CHE_INFRA_KUBERNETES_ENABLE__UNSUPPORTED__K8S]' to 'true'.`)
+		return true, nil
+	}
+
+	// check if DW exists on the cluster
 	devWorkspaceWebhookExists, err := deploy.Get(
 		deployContext,
 		client.ObjectKey{Name: DevWorkspaceWebhookName},
@@ -145,6 +140,7 @@ func ReconcileDevWorkspace(deployContext *deploy.DeployContext) (bool, error) {
 	}
 
 	if devWorkspaceWebhookExists {
+		// if DW exists then check if version matches
 		if err := checkWebTerminalSubscription(deployContext); err != nil {
 			return false, err
 		}
@@ -159,16 +155,28 @@ func ReconcileDevWorkspace(deployContext *deploy.DeployContext) (bool, error) {
 		}
 	}
 
-	for _, syncItem := range syncDwCheItems {
-		done, err := syncItem(deployContext)
-		if !util.IsTestMode() {
-			if !done {
-				return false, err
-			}
+	return true, nil
+}
+
+func isDevWorkspaceOperatorCSVExists(deployContext *deploy.DeployContext) bool {
+	// If clusterserviceversions resource doesn't exist in cluster DWO as well will not be present
+	if !util.HasK8SResourceObject(deployContext.ClusterAPI.DiscoveryClient, ClusterServiceVersionResourceName) {
+		return false
+	}
+
+	csvList := &operatorsv1alpha1.ClusterServiceVersionList{}
+	err := deployContext.ClusterAPI.Client.List(context.TODO(), csvList, &client.ListOptions{})
+	if err != nil {
+		return false
+	}
+
+	for _, csv := range csvList.Items {
+		if strings.Contains(csv.Name, DevWorkspaceCSVNameWithouVersion) {
+			return true
 		}
 	}
 
-	return true, nil
+	return false
 }
 
 func checkWebTerminalSubscription(deployContext *deploy.DeployContext) error {
@@ -211,60 +219,90 @@ func createDwNamespace(deployContext *deploy.DeployContext) (bool, error) {
 }
 
 func syncDwServiceAccount(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceServiceAccountFile, &corev1.ServiceAccount{})
+	return readAndSyncObject(deployContext, DevWorkspaceServiceAccountFile, &corev1.ServiceAccount{}, DevWorkspaceNamespace)
+}
+
+func syncDwService(deployContext *deploy.DeployContext) (bool, error) {
+	return readAndSyncObject(deployContext, DevWorkspaceServiceFile, &corev1.Service{}, DevWorkspaceNamespace)
 }
 
 func syncDwRole(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceRoleFile, &rbacv1.Role{})
+	return readAndSyncObject(deployContext, DevWorkspaceRoleFile, &rbacv1.Role{}, DevWorkspaceNamespace)
 }
 
 func syncDwRoleBinding(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceRoleBindingFile, &rbacv1.RoleBinding{})
+	return readAndSyncObject(deployContext, DevWorkspaceRoleBindingFile, &rbacv1.RoleBinding{}, DevWorkspaceNamespace)
 }
 
 func syncDwClusterRoleBinding(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceClusterRoleBindingFile, &rbacv1.ClusterRoleBinding{})
+	return readAndSyncObject(deployContext, DevWorkspaceClusterRoleBindingFile, &rbacv1.ClusterRoleBinding{}, "")
 }
 
 func syncDwProxyClusterRoleBinding(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceProxyClusterRoleBindingFile, &rbacv1.ClusterRoleBinding{})
+	return readAndSyncObject(deployContext, DevWorkspaceProxyClusterRoleBindingFile, &rbacv1.ClusterRoleBinding{}, "")
 }
 
 func syncDwClusterRole(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceClusterRoleFile, &rbacv1.ClusterRole{})
+	return readAndSyncObject(deployContext, DevWorkspaceClusterRoleFile, &rbacv1.ClusterRole{}, "")
 }
 
 func syncDwProxyClusterRole(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceProxyClusterRoleFile, &rbacv1.ClusterRole{})
+	return readAndSyncObject(deployContext, DevWorkspaceProxyClusterRoleFile, &rbacv1.ClusterRole{}, "")
 }
 
 func syncDwViewWorkspacesClusterRole(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceViewWorkspacesClusterRoleFile, &rbacv1.ClusterRole{})
+	return readAndSyncObject(deployContext, DevWorkspaceViewWorkspacesClusterRoleFile, &rbacv1.ClusterRole{}, "")
 }
 
 func syncDwEditWorkspacesClusterRole(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceEditWorkspacesClusterRoleFile, &rbacv1.ClusterRole{})
+	return readAndSyncObject(deployContext, DevWorkspaceEditWorkspacesClusterRoleFile, &rbacv1.ClusterRole{}, "")
 }
 
 func syncDwWorkspaceRoutingCRD(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceWorkspaceRoutingCRDFile, &apiextensionsv1.CustomResourceDefinition{})
+	return readAndSyncObject(deployContext, DevWorkspaceWorkspaceRoutingCRDFile, &apiextensionsv1.CustomResourceDefinition{}, "")
 }
 
 func syncDwTemplatesCRD(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceTemplatesCRDFile, &apiextensionsv1.CustomResourceDefinition{})
+	return readAndSyncObject(deployContext, DevWorkspaceTemplatesCRDFile, &apiextensionsv1.CustomResourceDefinition{}, "")
 }
 
 func syncDwCRD(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCRDFile, &apiextensionsv1.CustomResourceDefinition{})
+	return readAndSyncObject(deployContext, DevWorkspaceCRDFile, &apiextensionsv1.CustomResourceDefinition{}, "")
+}
+
+func syncDwIssuer(deployContext *deploy.DeployContext) (bool, error) {
+	if !util.IsOpenShift {
+		// We're using unstructured to not require a direct dependency on the cert-manager
+		// This will cause a failure if cert-manager is not installed, which we're ok with
+		// Also, our Sync functionality requires the scheme to have the type we want to persist registered.
+		// In case of cert-manager objects, we don't want that because we would have to depend
+		// on cert manager, which would require us to also update operator-sdk version because cert-manager
+		// uses extension/v1 objects. So, we have to go the unstructured way here...
+		return readAndSyncUnstructured(deployContext, DevWorkspaceIssuerFile)
+	}
+	return true, nil
+}
+
+func syncDwCertificate(deployContext *deploy.DeployContext) (bool, error) {
+	if !util.IsOpenShift {
+		// We're using unstructured to not require a direct dependency on the cert-manager
+		// This will cause a failure if cert-manager is not installed, which we're ok with
+		// Also, our Sync functionality requires the scheme to have the type we want to persist registered.
+		// In case of cert-manager objects, we don't want that because we would have to depend
+		// on cert manager, which would require us to also update operator-sdk version because cert-manager
+		// uses extension/v1 objects. So, we have to go the unstructured way here...
+		return readAndSyncUnstructured(deployContext, DevWorkspaceCertificateFile)
+	}
+	return true, nil
 }
 
 func syncDwConfigMap(deployContext *deploy.DeployContext) (bool, error) {
-	devObject, err := readK8SObject(DevWorkspaceConfigMapFile, &corev1.ConfigMap{})
+	obj2sync, err := readK8SObject(DevWorkspaceConfigMapFile, &corev1.ConfigMap{})
 	if err != nil {
 		return false, err
 	}
 
-	configMap := devObject.obj.(*corev1.ConfigMap)
+	configMap := obj2sync.obj.(*corev1.ConfigMap)
 	// Remove when DevWorkspace controller should not care about DWR base host #373 https://github.com/devfile/devworkspace-operator/issues/373
 	if !util.IsOpenShift {
 		if configMap.Data == nil {
@@ -273,113 +311,55 @@ func syncDwConfigMap(deployContext *deploy.DeployContext) (bool, error) {
 		configMap.Data["devworkspace.routing.cluster_host_suffix"] = deployContext.CheCluster.Spec.K8s.IngressDomain
 	}
 
-	return syncObject(deployContext, devObject)
+	return syncObject(deployContext, obj2sync, DevWorkspaceNamespace)
 }
 
 func syncDwDeployment(deployContext *deploy.DeployContext) (bool, error) {
-	devObject, err := readK8SObject(DevWorkspaceDeploymentFile, &appsv1.Deployment{})
+	obj2sync, err := readK8SObject(DevWorkspaceDeploymentFile, &appsv1.Deployment{})
 	if err != nil {
 		return false, err
 	}
 
 	devworkspaceControllerImage := util.GetValue(deployContext.CheCluster.Spec.DevWorkspace.ControllerImage, deploy.DefaultDevworkspaceControllerImage(deployContext.CheCluster))
-	deploymentObject := devObject.obj.(*appsv1.Deployment)
+	deploymentObject := obj2sync.obj.(*appsv1.Deployment)
 	deploymentObject.Spec.Template.Spec.Containers[0].Image = devworkspaceControllerImage
 
-	return syncObject(deployContext, devObject)
+	return syncObject(deployContext, obj2sync, DevWorkspaceNamespace)
 }
 
-func createDwCheNamespace(deployContext *deploy.DeployContext) (bool, error) {
-	namespace := &corev1.Namespace{
-		TypeMeta: metav1.TypeMeta{
-			Kind:       "Namespace",
-			APIVersion: corev1.SchemeGroupVersion.String(),
-		},
-		ObjectMeta: metav1.ObjectMeta{
-			Name: DevWorkspaceCheNamespace,
-		},
-		Spec: corev1.NamespaceSpec{},
+func readAndSyncObject(deployContext *deploy.DeployContext, yamlFile string, obj interface{}, namespace string) (bool, error) {
+	obj2sync, err := readK8SObject(yamlFile, obj)
+	if err != nil {
+		return false, err
 	}
 
-	return deploy.CreateIfNotExists(deployContext, namespace)
+	return syncObject(deployContext, obj2sync, namespace)
 }
 
-func syncDwCheServiceAccount(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheServiceAccountFile, &corev1.ServiceAccount{})
-}
-
-func syncDwCheClusterRole(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheClusterRoleFile, &rbacv1.ClusterRole{})
-}
-
-func syncDwCheProxyClusterRole(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheProxyClusterRoleFile, &rbacv1.ClusterRole{})
-}
-
-func syncDwCheMetricsClusterRole(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheMetricsReaderClusterRoleFile, &rbacv1.ClusterRole{})
-}
-
-func syncDwCheLeaderRole(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheRoleFile, &rbacv1.Role{})
-}
-
-func syncDwCheLeaderRoleBinding(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheRoleBindingFile, &rbacv1.RoleBinding{})
-}
-
-func syncDwCheProxyRoleBinding(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheProxyClusterRoleBindingFile, &rbacv1.ClusterRoleBinding{})
-}
-
-func syncDwCheRoleBinding(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheClusterRoleBindingFile, &rbacv1.ClusterRoleBinding{})
-}
-
-func syncDwCheCRD(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheManagersCRDFile, &apiextensionsv1.CustomResourceDefinition{})
-}
-
-func syncDwCheConfigMap(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheConfigMapFile, &corev1.ConfigMap{})
-}
-
-func syncDwCheMetricsService(deployContext *deploy.DeployContext) (bool, error) {
-	return readAndSyncObject(deployContext, DevWorkspaceCheMetricsServiceFile, &corev1.Service{})
-}
-
-func synDwCheCR(deployContext *deploy.DeployContext) (bool, error) {
-	// We want to create a default CheManager instance to be able to configure the che-specific
-	// parts of the installation, but at the same time we don't want to add a dependency on
-	// devworkspace-che-operator. Note that this way of initializing will probably see changes
-	// once we figure out https://github.com/eclipse/che/issues/19220
-
-	// Wait until CRD for CheManager is created
-	if !util.HasK8SResourceObject(deployContext.ClusterAPI.DiscoveryClient, CheManagerResourcename) {
-		return false, nil
-	}
-
+func readAndSyncUnstructured(deployContext *deploy.DeployContext, yamlFile string) (bool, error) {
 	obj := &unstructured.Unstructured{}
-	obj.SetGroupVersionKind(schema.GroupVersionKind{Group: "che.eclipse.org", Version: "v1alpha1", Kind: "CheManager"})
-	err := deployContext.ClusterAPI.Client.Get(context.TODO(), client.ObjectKey{Name: "devworkspace-che", Namespace: DevWorkspaceCheNamespace}, obj)
+	obj2sync, err := readK8SObject(yamlFile, obj)
+	if err != nil {
+		return false, err
+	}
+
+	return createUnstructured(deployContext, obj2sync.obj.(*unstructured.Unstructured))
+}
+
+func createUnstructured(deployContext *deploy.DeployContext, obj *unstructured.Unstructured) (bool, error) {
+	check := &unstructured.Unstructured{}
+	check.SetGroupVersionKind(obj.GroupVersionKind())
+
+	err := deployContext.ClusterAPI.Client.Get(context.TODO(), client.ObjectKey{Name: obj.GetName(), Namespace: obj.GetNamespace()}, obj)
 	if err != nil {
 		if apierrors.IsNotFound(err) {
-			obj = nil
+			check = nil
 		} else {
 			return false, err
 		}
 	}
 
-	if obj == nil {
-		obj := &unstructured.Unstructured{}
-		obj.SetGroupVersionKind(schema.GroupVersionKind{
-			Group:   "che.eclipse.org",
-			Version: "v1alpha1",
-			Kind:    "CheManager",
-		})
-		obj.SetName("devworkspace-che")
-		obj.SetNamespace(DevWorkspaceCheNamespace)
-
+	if check == nil {
 		err = deployContext.ClusterAPI.Client.Create(context.TODO(), obj)
 		if err != nil {
 			if apierrors.IsAlreadyExists(err) {
@@ -392,29 +372,9 @@ func synDwCheCR(deployContext *deploy.DeployContext) (bool, error) {
 	return true, nil
 }
 
-func synDwCheDeployment(deployContext *deploy.DeployContext) (bool, error) {
-	devObject, err := readK8SObject(DevWorkspaceCheDeploymentFile, &appsv1.Deployment{})
-	if err != nil {
-		return false, err
-	}
+func syncObject(deployContext *deploy.DeployContext, obj2sync *Object2Sync, namespace string) (bool, error) {
+	obj2sync.obj.SetNamespace(namespace)
 
-	devworkspaceCheOperatorImage := deploy.DefaultDevworkspaceCheOperatorImage(deployContext.CheCluster)
-	deploymentObject := devObject.obj.(*appsv1.Deployment)
-	deploymentObject.Spec.Template.Spec.Containers[0].Image = devworkspaceCheOperatorImage
-
-	return syncObject(deployContext, devObject)
-}
-
-func readAndSyncObject(deployContext *deploy.DeployContext, yamlFile string, obj interface{}) (bool, error) {
-	obj2sync, err := readK8SObject(yamlFile, obj)
-	if err != nil {
-		return false, err
-	}
-
-	return syncObject(deployContext, obj2sync)
-}
-
-func syncObject(deployContext *deploy.DeployContext, obj2sync *Object2Sync) (bool, error) {
 	runtimeObject, ok := obj2sync.obj.(runtime.Object)
 	if !ok {
 		return false, fmt.Errorf("object %T is not a runtime.Object. Cannot sync it", runtimeObject)
@@ -501,10 +461,3 @@ func devWorkspaceTemplatesPath() string {
 	}
 	return KubernetesDevWorkspaceTemplatesPath
 }
-
-func devWorkspaceCheTemplatesPath() string {
-	if util.IsOpenShift {
-		return OpenshiftDevWorkspaceCheTemplatesPath
-	}
-	return KubernetesDevWorkspaceCheTemplatesPath
-}
