@@ -14,73 +14,128 @@ package devworkspace
 import (
 	"context"
 
-	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	orgv1 "github.com/eclipse-che/che-operator/api/v1"
 	"github.com/eclipse-che/che-operator/pkg/deploy"
 	"github.com/eclipse-che/che-operator/pkg/util"
 	operatorsv1alpha1 "github.com/operator-framework/api/pkg/operators/v1alpha1"
 	admissionregistrationv1 "k8s.io/api/admissionregistration/v1"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
-	"k8s.io/apimachinery/pkg/runtime/schema"
 	fakeDiscovery "k8s.io/client-go/discovery/fake"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 
 	"testing"
 )
 
+const (
+	DevWorkspaceCSVName = "devworkspace-operator.v0.6.0"
+)
+
 func TestReconcileDevWorkspace(t *testing.T) {
-	cheCluster := &orgv1.CheCluster{
-		ObjectMeta: metav1.ObjectMeta{
-			Namespace: "eclipse-che",
-		},
-		Spec: orgv1.CheClusterSpec{
-			DevWorkspace: orgv1.CheClusterSpecDevWorkspace{
-				Enable: true,
-			},
-			Auth: orgv1.CheClusterSpecAuth{
-				OpenShiftoAuth: util.NewBoolPointer(true),
-			},
-			Server: orgv1.CheClusterSpecServer{
-				ServerExposureStrategy: "single-host",
-			},
-		},
+	type testCase struct {
+		name         string
+		IsOpenShift  bool
+		IsOpenShift4 bool
+		cheCluster   *orgv1.CheCluster
 	}
 
-	deployContext := deploy.GetTestDeployContext(cheCluster, []runtime.Object{})
-	deployContext.ClusterAPI.Scheme.AddKnownTypes(operatorsv1alpha1.SchemeGroupVersion, &operatorsv1alpha1.Subscription{})
-	deployContext.ClusterAPI.DiscoveryClient.(*fakeDiscovery.FakeDiscovery).Fake.Resources = []*metav1.APIResourceList{
+	testCases := []testCase{
 		{
-			APIResources: []metav1.APIResource{
-				{Name: CheManagerResourcename},
+			name: "Reconcile DevWorkspace on OpenShift",
+			cheCluster: &orgv1.CheCluster{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace: "eclipse-che",
+				},
+				Spec: orgv1.CheClusterSpec{
+					DevWorkspace: orgv1.CheClusterSpecDevWorkspace{
+						Enable: true,
+					},
+					Auth: orgv1.CheClusterSpecAuth{
+						OpenShiftoAuth: util.NewBoolPointer(true),
+					},
+					Server: orgv1.CheClusterSpecServer{
+						ServerExposureStrategy: "single-host",
+					},
+				},
 			},
+			IsOpenShift:  true,
+			IsOpenShift4: true,
+		},
+		{
+			name: "Reconcile DevWorkspace on K8S multi-host",
+			cheCluster: &orgv1.CheCluster{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace: "eclipse-che",
+				},
+				Spec: orgv1.CheClusterSpec{
+					DevWorkspace: orgv1.CheClusterSpecDevWorkspace{
+						Enable: true,
+					},
+					Auth: orgv1.CheClusterSpecAuth{
+						OpenShiftoAuth: util.NewBoolPointer(true),
+					},
+					Server: orgv1.CheClusterSpecServer{
+						ServerExposureStrategy: "multi-host",
+					},
+					K8s: orgv1.CheClusterSpecK8SOnly{
+						IngressDomain: "che.domain",
+					},
+				},
+			},
+			IsOpenShift:  false,
+			IsOpenShift4: false,
+		},
+		{
+			name: "Reconcile DevWorkspace on K8S single-host",
+			cheCluster: &orgv1.CheCluster{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace: "eclipse-che",
+				},
+				Spec: orgv1.CheClusterSpec{
+					DevWorkspace: orgv1.CheClusterSpecDevWorkspace{
+						Enable: true,
+					},
+					Auth: orgv1.CheClusterSpecAuth{
+						OpenShiftoAuth: util.NewBoolPointer(true),
+					},
+					Server: orgv1.CheClusterSpecServer{
+						ServerExposureStrategy: "single-host",
+					},
+					K8s: orgv1.CheClusterSpecK8SOnly{
+						IngressDomain: "che.domain",
+					},
+				},
+			},
+			IsOpenShift:  false,
+			IsOpenShift4: false,
 		},
 	}
 
-	util.IsOpenShift4 = true
-	done, err := ReconcileDevWorkspace(deployContext)
+	for _, testCase := range testCases {
+		t.Run(testCase.name, func(t *testing.T) {
+			deployContext := deploy.GetTestDeployContext(testCase.cheCluster, []runtime.Object{})
+			deployContext.ClusterAPI.Scheme.AddKnownTypes(operatorsv1alpha1.SchemeGroupVersion, &operatorsv1alpha1.Subscription{})
+			deployContext.ClusterAPI.Scheme.AddKnownTypes(operatorsv1alpha1.SchemeGroupVersion, &operatorsv1alpha1.ClusterServiceVersion{})
+			deployContext.ClusterAPI.DiscoveryClient.(*fakeDiscovery.FakeDiscovery).Fake.Resources = []*metav1.APIResourceList{
+				{
+					APIResources: []metav1.APIResource{
+						{Name: CheManagerResourcename},
+					},
+				},
+			}
 
-	if err != nil {
-		t.Fatalf("Error: %v", err)
+			util.IsOpenShift = testCase.IsOpenShift
+			util.IsOpenShift4 = testCase.IsOpenShift4
+			done, err := ReconcileDevWorkspace(deployContext)
+			if err != nil {
+				t.Fatalf("Error: %v", err)
+			}
+			if !done {
+				t.Fatalf("Dev Workspace operator has not been provisioned")
+			}
+		})
 	}
-
-	if !done {
-		t.Fatalf("Dev Workspace operator has not been provisioned")
-	}
-
-	t.Run("defaultCheManagerDeployed", func(t *testing.T) {
-		obj := &unstructured.Unstructured{}
-		obj.SetGroupVersionKind(schema.GroupVersionKind{Group: "che.eclipse.org", Version: "v1alpha1", Kind: "CheManager"})
-		err := deployContext.ClusterAPI.Client.Get(context.TODO(), client.ObjectKey{Name: "devworkspace-che", Namespace: DevWorkspaceCheNamespace}, obj)
-		if err != nil {
-			t.Fatalf("Should have found a CheManager with default config but got an error: %s", err)
-		}
-
-		if obj.GetName() != "devworkspace-che" {
-			t.Fatalf("Should have found a CheManager with default config but found: %s", obj.GetName())
-		}
-	})
 }
 
 func TestReconcileDevWorkspaceShouldThrowErrorIfWebTerminalSubscriptionExists(t *testing.T) {
@@ -124,6 +179,7 @@ func TestReconcileDevWorkspaceShouldThrowErrorIfWebTerminalSubscriptionExists(t
 		},
 	}
 
+	util.IsOpenShift = true
 	util.IsOpenShift4 = true
 	_, err := ReconcileDevWorkspace(deployContext)
 
@@ -132,6 +188,60 @@ func TestReconcileDevWorkspaceShouldThrowErrorIfWebTerminalSubscriptionExists(t
 	}
 }
 
+func TestReconcileDevWorkspaceCheckIfCSVExists(t *testing.T) {
+	cheCluster := &orgv1.CheCluster{
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: "eclipse-che",
+		},
+		Spec: orgv1.CheClusterSpec{
+			DevWorkspace: orgv1.CheClusterSpecDevWorkspace{
+				Enable: true,
+			},
+			Auth: orgv1.CheClusterSpecAuth{
+				OpenShiftoAuth: util.NewBoolPointer(true),
+			},
+			Server: orgv1.CheClusterSpecServer{
+				ServerExposureStrategy: "single-host",
+			},
+		},
+	}
+	devWorkspaceCSV := &operatorsv1alpha1.ClusterServiceVersion{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      DevWorkspaceCSVName,
+			Namespace: "openshift-operators",
+		},
+		Spec: operatorsv1alpha1.ClusterServiceVersionSpec{},
+	}
+
+	deployContext := deploy.GetTestDeployContext(cheCluster, []runtime.Object{})
+	deployContext.ClusterAPI.Scheme.AddKnownTypes(operatorsv1alpha1.SchemeGroupVersion, &operatorsv1alpha1.ClusterServiceVersion{})
+	deployContext.ClusterAPI.Scheme.AddKnownTypes(operatorsv1alpha1.SchemeGroupVersion, &operatorsv1alpha1.ClusterServiceVersionList{})
+	deployContext.ClusterAPI.Client.Create(context.TODO(), devWorkspaceCSV)
+	deployContext.ClusterAPI.DiscoveryClient.(*fakeDiscovery.FakeDiscovery).Fake.Resources = []*metav1.APIResourceList{
+		{
+			APIResources: []metav1.APIResource{
+				{
+					Name: ClusterServiceVersionResourceName,
+				},
+			},
+		},
+	}
+
+	util.IsOpenShift = true
+	util.IsOpenShift4 = true
+	reconciled, _ := ReconcileDevWorkspace(deployContext)
+
+	if !reconciled {
+		t.Fatalf("Failed to reconcile DevWorkspace")
+	}
+
+	// Get Devworkspace namespace. If error is thrown means devworkspace is not anymore installed if CSV is detected
+	err := deployContext.ClusterAPI.Client.Get(context.TODO(), client.ObjectKey{Name: DevWorkspaceNamespace}, &corev1.Namespace{})
+	if err == nil {
+		t.Fatal("Failed to reconcile DevWorkspace when DWO CSV is exptected to be created")
+	}
+}
+
 func TestShouldSyncNewObject(t *testing.T) {
 	deployContext := deploy.GetTestDeployContext(nil, []runtime.Object{})
 
@@ -142,7 +252,7 @@ func TestShouldSyncNewObject(t *testing.T) {
 	}
 
 	// tries to sync a new object
-	done, err := syncObject(deployContext, obj2sync)
+	done, err := syncObject(deployContext, obj2sync, "eclipse-che")
 	if err != nil {
 		t.Fatalf("Failed to sync object: %v", err)
 	} else if !done {
@@ -194,7 +304,7 @@ func TestShouldSyncObjectIfItWasCreatedByAnotherOriginHashDifferent(t *testing.T
 		obj:     newObject,
 		hash256: "hash",
 	}
-	_, err := syncObject(deployContext, obj2sync)
+	_, err := syncObject(deployContext, obj2sync, "eclipse-che")
 	if err != nil {
 		t.Fatalf("Failed to sync object: %v", err)
 	}
@@ -238,7 +348,7 @@ func TestShouldSyncObjectIfItWasCreatedBySameOriginHashDifferent(t *testing.T) {
 	}
 
 	// tries to sync object with a new
-	_, err := syncObject(deployContext, obj2sync)
+	_, err := syncObject(deployContext, obj2sync, "eclipse-che")
 	if err != nil {
 		t.Fatalf("Failed to sync object: %v", err)
 	}
@@ -303,7 +413,7 @@ func TestShouldNotSyncObjectIfThereIsAnotherCheCluster(t *testing.T) {
 		obj:     newObject,
 		hash256: "hash-1",
 	}
-	done, err := syncObject(deployContext, obj2sync)
+	done, err := syncObject(deployContext, obj2sync, "eclipse-che")
 	if err != nil {
 		t.Fatalf("Failed to sync object: %v", err)
 	} else if !done {
@@ -347,7 +457,7 @@ func TestShouldNotSyncObjectIfHashIsEqual(t *testing.T) {
 		obj:     newObject,
 		hash256: "hash",
 	}
-	done, err := syncObject(deployContext, obj2sync)
+	done, err := syncObject(deployContext, obj2sync, "eclipse-che")
 	if err != nil {
 		t.Fatalf("Failed to sync object: %v", err)
 	} else if !done {
