@@ -13,13 +13,17 @@ package gateway
 
 import (
 	"context"
+	"encoding/base64"
+	"fmt"
+	"io/ioutil"
 	"strconv"
 
+	"github.com/sirupsen/logrus"
+
 	"github.com/eclipse-che/che-operator/pkg/deploy"
 
-	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	orgv1 "github.com/eclipse-che/che-operator/api/v1"
 	"github.com/eclipse-che/che-operator/pkg/util"
-	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -39,17 +43,15 @@ const (
 
 	gatewayServerConfigName    = "che-gateway-route-server"
 	gatewayConfigComponentName = "che-gateway-config"
+	gatewayOauthSecretName     = "che-gateway-oauth-secret"
 )
 
 var (
 	serviceAccountDiffOpts = cmpopts.IgnoreFields(corev1.ServiceAccount{}, "TypeMeta", "ObjectMeta", "Secrets", "ImagePullSecrets")
 	roleDiffOpts           = cmpopts.IgnoreFields(rbac.Role{}, "TypeMeta", "ObjectMeta")
 	roleBindingDiffOpts    = cmpopts.IgnoreFields(rbac.RoleBinding{}, "TypeMeta", "ObjectMeta")
-	serviceDiffOpts        = cmp.Options{
-		cmpopts.IgnoreFields(corev1.Service{}, "TypeMeta", "ObjectMeta", "Status"),
-		cmpopts.IgnoreFields(corev1.ServiceSpec{}, "ClusterIP"),
-	}
-	configMapDiffOpts = cmpopts.IgnoreFields(corev1.ConfigMap{}, "TypeMeta", "ObjectMeta")
+	configMapDiffOpts      = cmpopts.IgnoreFields(corev1.ConfigMap{}, "TypeMeta", "ObjectMeta")
+	secretDiffOpts         = cmpopts.IgnoreFields(corev1.Secret{}, "TypeMeta", "ObjectMeta")
 )
 
 // SyncGatewayToCluster installs or deletes the gateway based on the custom resource configuration
@@ -79,6 +81,28 @@ func syncAll(deployContext *deploy.DeployContext) error {
 		return err
 	}
 
+	if util.IsNativeUserModeEnabled(instance) {
+		if oauthSecret, err := getGatewaySecretSpec(deployContext); err == nil {
+			if _, err := deploy.Sync(deployContext, oauthSecret, secretDiffOpts); err != nil {
+				return err
+			}
+			oauthProxyConfig := getGatewayOauthProxyConfigSpec(instance, string(oauthSecret.Data["cookie_secret"]))
+			if _, err := deploy.Sync(deployContext, &oauthProxyConfig, configMapDiffOpts); err != nil {
+				return err
+			}
+		} else {
+			return err
+		}
+
+		if headerRewritePluginConfig, err := getGatewayHeaderRewritePluginConfigSpec(instance); err == nil {
+			if _, err := deploy.Sync(deployContext, headerRewritePluginConfig, configMapDiffOpts); err != nil {
+				return err
+			}
+		} else {
+			return err
+		}
+	}
+
 	traefikConfig := getGatewayTraefikConfigSpec(instance)
 	if _, err := deploy.Sync(deployContext, &traefikConfig, configMapDiffOpts); err != nil {
 		return err
@@ -90,7 +114,7 @@ func syncAll(deployContext *deploy.DeployContext) error {
 	}
 
 	service := getGatewayServiceSpec(instance)
-	if _, err := deploy.Sync(deployContext, &service, serviceDiffOpts); err != nil {
+	if _, err := deploy.Sync(deployContext, &service, deploy.ServiceDefaultDiffOpts); err != nil {
 		return err
 	}
 
@@ -102,6 +126,39 @@ func syncAll(deployContext *deploy.DeployContext) error {
 	return nil
 }
 
+func getGatewaySecretSpec(deployContext *deploy.DeployContext) (*corev1.Secret, error) {
+	secret := &corev1.Secret{}
+	exists, err := deploy.GetNamespacedObject(deployContext, gatewayOauthSecretName, secret)
+	if err == nil && exists {
+		if _, ok := secret.Data["cookie_secret"]; !ok {
+			logrus.Info("che-gateway-secret found, but does not contain `cookie_secret` value. Regenerating...")
+			return generateOauthSecretSpec(deployContext), nil
+		}
+		return secret, nil
+	} else if err == nil && !exists {
+		return generateOauthSecretSpec(deployContext), nil
+	} else {
+		return nil, err
+	}
+}
+
+func generateOauthSecretSpec(deployContext *deploy.DeployContext) *corev1.Secret {
+	return &corev1.Secret{
+		TypeMeta: metav1.TypeMeta{
+			APIVersion: corev1.SchemeGroupVersion.String(),
+			Kind:       "Secret",
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      gatewayOauthSecretName,
+			Namespace: deployContext.CheCluster.Namespace,
+			Labels:    deploy.GetLabels(deployContext.CheCluster, GatewayServiceName),
+		},
+		Data: map[string][]byte{
+			"cookie_secret": generateRandomCookieSecret(),
+		},
+	}
+}
+
 func deleteAll(deployContext *deploy.DeployContext) error {
 	instance := deployContext.CheCluster
 	clusterAPI := deployContext.ClusterAPI
@@ -188,6 +245,7 @@ func delete(clusterAPI deploy.ClusterAPI, obj metav1.Object) error {
 // new configuration for workspaces, so the name should not resemble any of the names created by the Che server.
 func GetGatewayRouteConfig(deployContext *deploy.DeployContext, component string, serviceName string, pathPrefix string, priority int, internalUrl string, stripPrefix bool) corev1.ConfigMap {
 	pathRewrite := pathPrefix != "/" && stripPrefix
+	nativeUser := util.IsNativeUserModeEnabled(deployContext.CheCluster)
 
 	data := `---
 http:
@@ -195,11 +253,16 @@ http:
     ` + serviceName + `:
       rule: "PathPrefix(` + "`" + pathPrefix + "`" + `)"
       service: ` + serviceName + `
-      priority: ` + strconv.Itoa(priority)
+      priority: ` + strconv.Itoa(priority) + `
+      middlewares: `
+
+	if nativeUser {
+		data += `
+      - "` + serviceName + `-header"`
+	}
 
 	if pathRewrite {
 		data += `
-      middlewares:
       - "` + serviceName + `"`
 	}
 
@@ -208,11 +271,20 @@ http:
     ` + serviceName + `:
       loadBalancer:
         servers:
-        - url: '` + internalUrl + `'`
+        - url: '` + internalUrl + `'
+  middlewares:`
+	if nativeUser {
+		data += `
+    ` + serviceName + `-header:
+      plugin:
+        header-rewrite:
+          from: X-Forwarded-Access-Token
+          to: Authorization
+          prefix: 'Bearer '`
+	}
 
 	if pathRewrite {
 		data += `
-  middlewares:
     ` + serviceName + `:
       stripPrefix:
         prefixes:
@@ -318,7 +390,107 @@ func getGatewayRoleBindingSpec(instance *orgv1.CheCluster) rbac.RoleBinding {
 	}
 }
 
+func getGatewayOauthProxyConfigSpec(instance *orgv1.CheCluster, cookieSecret string) corev1.ConfigMap {
+	return corev1.ConfigMap{
+		TypeMeta: metav1.TypeMeta{
+			APIVersion: corev1.SchemeGroupVersion.String(),
+			Kind:       "ConfigMap",
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      "che-gateway-config-oauth-proxy",
+			Namespace: instance.Namespace,
+			Labels:    deploy.GetLabels(instance, GatewayServiceName),
+		},
+		Data: map[string]string{
+			"oauth-proxy.cfg": fmt.Sprintf(`
+http_address = ":8080"
+https_address = ""
+provider = "openshift"
+redirect_url = "https://%s/oauth/callback"
+upstreams = [
+	"http://127.0.0.1:8081/"
+]
+client_id = "%s"
+client_secret = "%s"
+scope = "user:full"
+openshift_service_account = "%s"
+cookie_secret = "%s"
+email_domains = "*"
+cookie_httponly = false
+pass_access_token = true
+skip_provider_button = true`, instance.Spec.Server.CheHost, instance.Spec.Auth.OAuthClientName, instance.Spec.Auth.OAuthSecret, GatewayServiceName, cookieSecret),
+		},
+	}
+}
+
+func generateRandomCookieSecret() []byte {
+	return []byte(base64.StdEncoding.EncodeToString([]byte(util.GeneratePasswd(16))))
+}
+
+func getGatewayHeaderRewritePluginConfigSpec(instance *orgv1.CheCluster) (*corev1.ConfigMap, error) {
+	headerRewrite, err := ioutil.ReadFile("/tmp/header-rewrite-traefik-plugin/headerRewrite.go")
+	if err != nil {
+		if !util.IsTestMode() {
+			return nil, err
+		}
+	}
+	pluginMeta, err := ioutil.ReadFile("/tmp/header-rewrite-traefik-plugin/.traefik.yml")
+	if err != nil {
+		if !util.IsTestMode() {
+			return nil, err
+		}
+	}
+
+	return &corev1.ConfigMap{
+		TypeMeta: metav1.TypeMeta{
+			APIVersion: corev1.SchemeGroupVersion.String(),
+			Kind:       "ConfigMap",
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      "che-gateway-config-header-rewrite-traefik-plugin",
+			Namespace: instance.Namespace,
+			Labels:    deploy.GetLabels(instance, GatewayServiceName),
+		},
+		Data: map[string]string{
+			"headerRewrite.go": string(headerRewrite),
+			".traefik.yml":     string(pluginMeta),
+		},
+	}, nil
+}
+
 func getGatewayTraefikConfigSpec(instance *orgv1.CheCluster) corev1.ConfigMap {
+	traefikPort := 8080
+	if util.IsNativeUserModeEnabled(instance) {
+		traefikPort = 8081
+	}
+	data := fmt.Sprintf(`
+entrypoints:
+  http:
+    address: ":%d"
+    forwardedHeaders:
+      insecure: true
+  sink:
+    address: ":8090"
+ping:
+  entryPoint: "sink"
+global:
+  checkNewVersion: false
+  sendAnonymousUsage: false
+providers:
+  file:
+    directory: "/dynamic-config"
+    watch: true
+log:
+  level: "INFO"`, traefikPort)
+
+	if util.IsNativeUserModeEnabled(instance) {
+		data += `
+experimental:
+  localPlugins:
+    header-rewrite:
+      moduleName: github.com/che-incubator/header-rewrite-traefik-plugin`
+	}
+
 	return corev1.ConfigMap{
 		TypeMeta: metav1.TypeMeta{
 			APIVersion: corev1.SchemeGroupVersion.String(),
@@ -330,37 +502,15 @@ func getGatewayTraefikConfigSpec(instance *orgv1.CheCluster) corev1.ConfigMap {
 			Labels:    deploy.GetLabels(instance, GatewayServiceName),
 		},
 		Data: map[string]string{
-			"traefik.yml": `
-entrypoints:
-  http:
-    address: ":8080"
-    forwardedHeaders:
-      insecure: true
-  https:
-    address: ":8443"
-    forwardedHeaders:
-      insecure: true
-global:
-  checkNewVersion: false
-  sendAnonymousUsage: false
-providers:
-  file:
-    directory: "/dynamic-config"
-    watch: true
-log:
-  level: "INFO"`,
+			"traefik.yml": data,
 		},
 	}
 }
 
 func getGatewayDeploymentSpec(instance *orgv1.CheCluster) appsv1.Deployment {
-	gatewayImage := util.GetValue(instance.Spec.Server.SingleHostGatewayImage, deploy.DefaultSingleHostGatewayImage(instance))
-	sidecarImage := util.GetValue(instance.Spec.Server.SingleHostGatewayConfigSidecarImage, deploy.DefaultSingleHostGatewayConfigSidecarImage(instance))
-	configLabelsMap := util.GetMapValue(instance.Spec.Server.SingleHostGatewayConfigMapLabels, deploy.DefaultSingleHostGatewayConfigMapLabels)
 	terminationGracePeriodSeconds := int64(10)
 
-	configLabels := labels.FormatLabels(configLabelsMap)
-	labels, labelsSelector := deploy.GetLabelsAndSelector(instance, GatewayServiceName)
+	deployLabels, labelsSelector := deploy.GetLabelsAndSelector(instance, GatewayServiceName)
 
 	return appsv1.Deployment{
 		TypeMeta: metav1.TypeMeta{
@@ -370,7 +520,7 @@ func getGatewayDeploymentSpec(instance *orgv1.CheCluster) appsv1.Deployment {
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      GatewayServiceName,
 			Namespace: instance.Namespace,
-			Labels:    labels,
+			Labels:    deployLabels,
 		},
 		Spec: appsv1.DeploymentSpec{
 			Selector: &metav1.LabelSelector{
@@ -381,81 +531,167 @@ func getGatewayDeploymentSpec(instance *orgv1.CheCluster) appsv1.Deployment {
 			},
 			Template: corev1.PodTemplateSpec{
 				ObjectMeta: metav1.ObjectMeta{
-					Labels: labels,
+					Labels: deployLabels,
 				},
 				Spec: corev1.PodSpec{
 					TerminationGracePeriodSeconds: &terminationGracePeriodSeconds,
 					ServiceAccountName:            GatewayServiceName,
 					RestartPolicy:                 corev1.RestartPolicyAlways,
-					Containers: []corev1.Container{
-						{
-							Name:            "gateway",
-							Image:           gatewayImage,
-							ImagePullPolicy: corev1.PullAlways,
-							VolumeMounts: []corev1.VolumeMount{
-								{
-									Name:      "static-config",
-									MountPath: "/etc/traefik",
-								},
-								{
-									Name:      "dynamic-config",
-									MountPath: "/dynamic-config",
-								},
-							},
-						},
-						{
-							Name:            "configbump",
-							Image:           sidecarImage,
-							ImagePullPolicy: corev1.PullAlways,
-							VolumeMounts: []corev1.VolumeMount{
-								{
-									Name:      "dynamic-config",
-									MountPath: "/dynamic-config",
-								},
-							},
-							Env: []corev1.EnvVar{
-								{
-									Name:  "CONFIG_BUMP_DIR",
-									Value: "/dynamic-config",
-								},
-								{
-									Name:  "CONFIG_BUMP_LABELS",
-									Value: configLabels,
-								},
-								{
-									Name: "CONFIG_BUMP_NAMESPACE",
-									ValueFrom: &corev1.EnvVarSource{
-										FieldRef: &corev1.ObjectFieldSelector{
-											APIVersion: "v1",
-											FieldPath:  "metadata.namespace",
-										},
-									},
-								},
-							},
-						},
-					},
-					Volumes: []corev1.Volume{
-						{
-							Name: "static-config",
-							VolumeSource: corev1.VolumeSource{
-								ConfigMap: &corev1.ConfigMapVolumeSource{
-									LocalObjectReference: corev1.LocalObjectReference{
-										Name: "che-gateway-config",
-									},
-								},
-							},
-						},
-						{
-							Name: "dynamic-config",
-							VolumeSource: corev1.VolumeSource{
-								EmptyDir: &corev1.EmptyDirVolumeSource{},
-							},
+					Containers:                    getContainersSpec(instance),
+					Volumes:                       getVolumesSpec(instance),
+				},
+			},
+		},
+	}
+}
+
+func getContainersSpec(instance *orgv1.CheCluster) []corev1.Container {
+	configLabelsMap := util.GetMapValue(instance.Spec.Server.SingleHostGatewayConfigMapLabels, deploy.DefaultSingleHostGatewayConfigMapLabels)
+	gatewayImage := util.GetValue(instance.Spec.Server.SingleHostGatewayImage, deploy.DefaultSingleHostGatewayImage(instance))
+	configSidecarImage := util.GetValue(instance.Spec.Server.SingleHostGatewayConfigSidecarImage, deploy.DefaultSingleHostGatewayConfigSidecarImage(instance))
+	authnImage := util.GetValue(instance.Spec.Auth.GatewayAuthenticationSidecarImage, deploy.DefaultGatewayAuthenticationSidecarImage(instance))
+	authzImage := util.GetValue(instance.Spec.Auth.GatewayAuthorizationSidecarImage, deploy.DefaultGatewayAuthorizationSidecarImage(instance))
+	configLabels := labels.FormatLabels(configLabelsMap)
+
+	containers := []corev1.Container{
+		{
+			Name:            "gateway",
+			Image:           gatewayImage,
+			ImagePullPolicy: corev1.PullAlways,
+			VolumeMounts:    getTraefikContainerVolumeMounts(instance),
+		},
+		{
+			Name:            "configbump",
+			Image:           configSidecarImage,
+			ImagePullPolicy: corev1.PullAlways,
+			VolumeMounts: []corev1.VolumeMount{
+				{
+					Name:      "dynamic-config",
+					MountPath: "/dynamic-config",
+				},
+			},
+			Env: []corev1.EnvVar{
+				{
+					Name:  "CONFIG_BUMP_DIR",
+					Value: "/dynamic-config",
+				},
+				{
+					Name:  "CONFIG_BUMP_LABELS",
+					Value: configLabels,
+				},
+				{
+					Name: "CONFIG_BUMP_NAMESPACE",
+					ValueFrom: &corev1.EnvVarSource{
+						FieldRef: &corev1.ObjectFieldSelector{
+							APIVersion: "v1",
+							FieldPath:  "metadata.namespace",
 						},
 					},
 				},
 			},
 		},
 	}
+
+	if util.IsNativeUserModeEnabled(instance) {
+		containers = append(containers,
+			corev1.Container{
+				Name:            "oauth-proxy",
+				Image:           authnImage,
+				ImagePullPolicy: corev1.PullAlways,
+				Args: []string{
+					"--config=/etc/oauth-proxy/oauth-proxy.cfg",
+				},
+				VolumeMounts: []corev1.VolumeMount{
+					{
+						Name:      "oauth-proxy-config",
+						MountPath: "/etc/oauth-proxy",
+					},
+				},
+				Ports: []corev1.ContainerPort{
+					{ContainerPort: 8080},
+				},
+			},
+			corev1.Container{
+				Name:            "kube-rbac-proxy",
+				Image:           authzImage,
+				ImagePullPolicy: corev1.PullAlways,
+				Args: []string{
+					"--insecure-listen-address=127.0.0.1:8089",
+					"--upstream=http://127.0.0.1:8090/ping",
+					"--logtostderr=true",
+				},
+			})
+	}
+
+	return containers
+}
+
+func getTraefikContainerVolumeMounts(instance *orgv1.CheCluster) []corev1.VolumeMount {
+	mounts := []corev1.VolumeMount{
+		{
+			Name:      "static-config",
+			MountPath: "/etc/traefik",
+		},
+		{
+			Name:      "dynamic-config",
+			MountPath: "/dynamic-config",
+		},
+	}
+	if util.IsNativeUserModeEnabled(instance) {
+		mounts = append(mounts, corev1.VolumeMount{
+			Name:      "header-rewrite-traefik-plugin",
+			MountPath: "/plugins-local/src/github.com/che-incubator/header-rewrite-traefik-plugin",
+		})
+	}
+
+	return mounts
+}
+
+func getVolumesSpec(instance *orgv1.CheCluster) []corev1.Volume {
+	volumes := []corev1.Volume{
+		{
+			Name: "static-config",
+			VolumeSource: corev1.VolumeSource{
+				ConfigMap: &corev1.ConfigMapVolumeSource{
+					LocalObjectReference: corev1.LocalObjectReference{
+						Name: "che-gateway-config",
+					},
+				},
+			},
+		},
+		{
+			Name: "dynamic-config",
+			VolumeSource: corev1.VolumeSource{
+				EmptyDir: &corev1.EmptyDirVolumeSource{},
+			},
+		},
+	}
+
+	if util.IsNativeUserModeEnabled(instance) {
+		volumes = append(volumes, corev1.Volume{
+			Name: "oauth-proxy-config",
+			VolumeSource: corev1.VolumeSource{
+				ConfigMap: &corev1.ConfigMapVolumeSource{
+					LocalObjectReference: corev1.LocalObjectReference{
+						Name: "che-gateway-config-oauth-proxy",
+					},
+				},
+			},
+		})
+
+		volumes = append(volumes, corev1.Volume{
+			Name: "header-rewrite-traefik-plugin",
+			VolumeSource: corev1.VolumeSource{
+				ConfigMap: &corev1.ConfigMapVolumeSource{
+					LocalObjectReference: corev1.LocalObjectReference{
+						Name: "che-gateway-config-header-rewrite-traefik-plugin",
+					},
+				},
+			},
+		})
+	}
+
+	return volumes
 }
 
 func getGatewayServiceSpec(instance *orgv1.CheCluster) corev1.Service {
@@ -480,12 +716,6 @@ func getGatewayServiceSpec(instance *orgv1.CheCluster) corev1.Service {
 					Protocol:   corev1.ProtocolTCP,
 					TargetPort: intstr.FromInt(8080),
 				},
-				{
-					Name:       "gateway-https",
-					Port:       8443,
-					Protocol:   corev1.ProtocolTCP,
-					TargetPort: intstr.FromInt(8443),
-				},
 			},
 		},
 	}
