@@ -17,7 +17,7 @@ import (
 
 	"github.com/google/go-cmp/cmp"
 
-	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	orgv1 "github.com/eclipse-che/che-operator/api/v1"
 	"k8s.io/api/extensions/v1beta1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -39,11 +39,11 @@ func TestIngressSpec(t *testing.T) {
 		ingressCustomSettings orgv1.IngressCustomSettings
 		expectedIngress       *v1beta1.Ingress
 	}
-
+	cheFlavor := getDefaultFromEnv("CHE_FLAVOR")
 	cheCluster := &orgv1.CheCluster{
 		ObjectMeta: metav1.ObjectMeta{
 			Namespace: "eclipse-che",
-			Name:      "eclipse-che",
+			Name:      cheFlavor,
 		},
 	}
 
@@ -51,10 +51,10 @@ func TestIngressSpec(t *testing.T) {
 		{
 			name:             "Test custom host",
 			ingressName:      "test",
-			ingressComponent: "test-component",
+			ingressComponent: DefaultCheFlavor(cheCluster),
 			ingressHost:      "test-host",
 			ingressPath:      "",
-			serviceName:      "che",
+			serviceName:      "che-host",
 			servicePort:      8080,
 			ingressCustomSettings: orgv1.IngressCustomSettings{
 				Labels:      "type=default",
@@ -66,7 +66,7 @@ func TestIngressSpec(t *testing.T) {
 					Namespace: "eclipse-che",
 					Labels: map[string]string{
 						"type":                         "default",
-						"app.kubernetes.io/component":  "test-component",
+						"app.kubernetes.io/component":  DefaultCheFlavor(cheCluster),
 						"app.kubernetes.io/instance":   DefaultCheFlavor(cheCluster),
 						"app.kubernetes.io/managed-by": DefaultCheFlavor(cheCluster) + "-operator",
 						"app.kubernetes.io/name":       DefaultCheFlavor(cheCluster),
@@ -77,7 +77,8 @@ func TestIngressSpec(t *testing.T) {
 						"nginx.ingress.kubernetes.io/proxy-connect-timeout": "3600",
 						"nginx.ingress.kubernetes.io/proxy-read-timeout":    "3600",
 						"nginx.ingress.kubernetes.io/ssl-redirect":          "false",
-						"annotation-key": "annotation-value",
+						"nginx.org/websocket-services":                      "che-host",
+						"annotation-key":                                    "annotation-value",
 					},
 				},
 				TypeMeta: metav1.TypeMeta{
@@ -93,7 +94,7 @@ func TestIngressSpec(t *testing.T) {
 									Paths: []v1beta1.HTTPIngressPath{
 										{
 											Backend: v1beta1.IngressBackend{
-												ServiceName: "che",
+												ServiceName: "che-host",
 												ServicePort: intstr.FromInt(8080),
 											},
 											Path: "/",
@@ -143,7 +144,7 @@ func TestSyncIngressToCluster(t *testing.T) {
 	}
 
 	actual := &v1beta1.Ingress{}
-	err = deployContext.ClusterAPI.Client.Get(context.TODO(), types.NamespacedName{Name: "test"}, actual)
+	err = deployContext.ClusterAPI.Client.Get(context.TODO(), types.NamespacedName{Name: "test", Namespace: "eclipse-che"}, actual)
 	if err != nil {
 		t.Fatalf("Failed to get ingress: %v", err)
 	}
