@@ -13,9 +13,10 @@ package deploy
 
 import (
 	"reflect"
+	"sort"
 	"strconv"
 
-	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	orgv1 "github.com/eclipse-che/che-operator/api/v1"
 	"github.com/eclipse-che/che-operator/pkg/util"
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
@@ -26,8 +27,10 @@ import (
 
 var ingressDiffOpts = cmp.Options{
 	cmpopts.IgnoreFields(v1beta1.Ingress{}, "TypeMeta", "Status"),
+	cmpopts.IgnoreFields(v1beta1.HTTPIngressPath{}, "PathType"),
 	cmp.Comparer(func(x, y metav1.ObjectMeta) bool {
-		return reflect.DeepEqual(x.Labels, y.Labels)
+		return reflect.DeepEqual(x.Labels, y.Labels) &&
+			x.Annotations[CheEclipseOrgManagedAnnotationsDigest] == y.Annotations[CheEclipseOrgManagedAnnotationsDigest]
 	}),
 }
 
@@ -60,13 +63,22 @@ func GetIngressSpec(
 	ingressCustomSettings orgv1.IngressCustomSettings,
 	component string) (ingressUrl string, i *v1beta1.Ingress) {
 
+	cheFlavor := DefaultCheFlavor(deployContext.CheCluster)
 	tlsSupport := deployContext.CheCluster.Spec.Server.TlsSupport
 	ingressStrategy := util.GetServerExposureStrategy(deployContext.CheCluster)
 	ingressDomain := deployContext.CheCluster.Spec.K8s.IngressDomain
+	tlsSecretName := deployContext.CheCluster.Spec.K8s.TlsSecretName
 	ingressClass := util.GetValue(deployContext.CheCluster.Spec.K8s.IngressClass, DefaultIngressClass)
 	labels := GetLabels(deployContext.CheCluster, component)
 	MergeLabels(labels, ingressCustomSettings.Labels)
 
+	if tlsSupport {
+		// for server and dashboard ingresses
+		if (component == cheFlavor || component == cheFlavor+"-dashboard") && deployContext.CheCluster.Spec.Server.CheHostTLSSecret != "" {
+			tlsSecretName = deployContext.CheCluster.Spec.Server.CheHostTLSSecret
+		}
+	}
+
 	if host == "" {
 		if ingressStrategy == "multi-host" {
 			host = component + "-" + deployContext.CheCluster.Namespace + "." + ingressDomain
@@ -75,13 +87,6 @@ func GetIngressSpec(
 		}
 	}
 
-	tlsSecretName := util.GetValue(deployContext.CheCluster.Spec.K8s.TlsSecretName, "")
-	if tlsSupport {
-		if component == DefaultCheFlavor(deployContext.CheCluster) && deployContext.CheCluster.Spec.Server.CheHostTLSSecret != "" {
-			tlsSecretName = deployContext.CheCluster.Spec.Server.CheHostTLSSecret
-		}
-	}
-
 	var endpointPath, ingressPath string
 	if path == "" {
 		endpointPath, ingressPath = evaluatePath(component, ingressStrategy)
@@ -99,6 +104,29 @@ func GetIngressSpec(
 	if ingressStrategy != "multi-host" && (component == DevfileRegistryName || component == PluginRegistryName) {
 		annotations["nginx.ingress.kubernetes.io/rewrite-target"] = "/$1"
 	}
+	for k, v := range ingressCustomSettings.Annotations {
+		annotations[k] = v
+	}
+
+	// add 'che.eclipse.org/managed-annotations-digest' annotation
+	// to store and compare annotations managed by operator only
+	annotationsKeys := make([]string, 0, len(annotations))
+	for k := range annotations {
+		annotationsKeys = append(annotationsKeys, k)
+	}
+	if len(annotationsKeys) > 0 {
+		sort.Strings(annotationsKeys)
+
+		data := ""
+		for _, k := range annotationsKeys {
+			data += k + ":" + annotations[k] + ","
+		}
+		if util.IsTestMode() {
+			annotations[CheEclipseOrgManagedAnnotationsDigest] = "0000"
+		} else {
+			annotations[CheEclipseOrgManagedAnnotationsDigest] = util.ComputeHash256([]byte(data))
+		}
+	}
 
 	ingress := &v1beta1.Ingress{
 		TypeMeta: metav1.TypeMeta{
@@ -133,12 +161,15 @@ func GetIngressSpec(
 		},
 	}
 
+	if component == cheFlavor {
+		// adds annotation, see details https://github.com/eclipse/che/issues/19434#issuecomment-810325262
+		ingress.ObjectMeta.Annotations["nginx.org/websocket-services"] = serviceName
+	}
+
 	if tlsSupport {
 		ingress.Spec.TLS = []v1beta1.IngressTLS{
 			{
-				Hosts: []string{
-					ingressDomain,
-				},
+				Hosts:      []string{host},
 				SecretName: tlsSecretName,
 			},
 		}
