@@ -13,10 +13,12 @@ package deploy
 
 import (
 	"context"
+	goerror "errors"
+	"strings"
 	"time"
 
 	chev1alpha1 "github.com/che-incubator/kubernetes-image-puller-operator/pkg/apis/che/v1alpha1"
-	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	orgv1 "github.com/eclipse-che/che-operator/api/v1"
 	"github.com/eclipse-che/che-operator/pkg/util"
 	operatorsv1 "github.com/operator-framework/api/pkg/operators/v1"
 	operatorsv1alpha1 "github.com/operator-framework/api/pkg/operators/v1alpha1"
@@ -25,12 +27,19 @@ import (
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/apimachinery/pkg/util/validation"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 )
 
 var imagePullerFinalizerName = "kubernetesimagepullers.finalizers.che.eclipse.org"
 
+// ImageAndName represents an image coupled with an image name.
+type ImageAndName struct {
+	name  string // image name (ex. my-image)
+	image string // image (ex. quay.io/test/abc)
+}
+
 // Reconcile the imagePuller section of the CheCluster CR.  If imagePuller.enable is set to true, install the Kubernetes Image Puller operator and create
 // a KubernetesImagePuller CR.  Add a finalizer to the CheCluster CR.  If false, remove the KubernetesImagePuller CR, uninstall the operator, and remove the finalizer.
 func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
@@ -45,7 +54,7 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 	// If the image puller should be installed but the APIServer doesn't know about PackageManifests/Subscriptions, log a warning and requeue
 	if ctx.CheCluster.Spec.ImagePuller.Enable && (!foundPackagesAPI || !foundOperatorsAPI) {
 		logrus.Infof("Couldn't find Operator Lifecycle Manager types to install the Kubernetes Image Puller Operator.  Please install Operator Lifecycle Manager to install the operator or disable the image puller by setting spec.imagePuller.enable to false.")
-		return reconcile.Result{Requeue: true}, nil
+		return reconcile.Result{RequeueAfter: time.Second}, nil
 	}
 
 	if ctx.CheCluster.Spec.ImagePuller.Enable {
@@ -54,7 +63,7 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 			if err != nil {
 				if errors.IsNotFound(err) {
 					logrus.Infof("There is no PackageManifest for the Kubernetes Image Puller Operator.  Install the Operator Lifecycle Manager and the Community Operators Catalog")
-					return reconcile.Result{Requeue: true}, nil
+					return reconcile.Result{RequeueAfter: time.Second}, nil
 				}
 				logrus.Errorf("Error getting packagemanifest: %v", err)
 				return reconcile.Result{}, err
@@ -66,7 +75,7 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 				return reconcile.Result{}, err
 			}
 			if createdOperatorGroup {
-				return reconcile.Result{Requeue: true}, nil
+				return reconcile.Result{RequeueAfter: time.Second}, nil
 			}
 			createdOperatorSubscription, err := CreateImagePullerSubscription(ctx, packageManifest)
 			if err != nil {
@@ -74,30 +83,15 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 				return reconcile.Result{}, err
 			}
 			if createdOperatorSubscription {
-				return reconcile.Result{Requeue: true}, nil
-			}
-			subscriptionsAreEqual, err := CompareExpectedSubscription(ctx, packageManifest)
-			if err != nil {
-				logrus.Infof("Error checking Subscription equality: %v", err)
-				return reconcile.Result{}, nil
-			}
-			// If the Subscription Spec changed for some reason, update it
-			if !subscriptionsAreEqual {
-				updatedOperatorSubscription := GetExpectedSubscription(ctx, packageManifest)
-				logrus.Infof("Updating Subscription")
-				err = ctx.ClusterAPI.NonCachedClient.Update(context.TODO(), updatedOperatorSubscription, &client.UpdateOptions{})
-				if err != nil {
-					logrus.Errorf("Error updating Subscription: %v", err)
-					return reconcile.Result{}, err
-				}
-				return reconcile.Result{Requeue: true}, nil
+				return reconcile.Result{RequeueAfter: time.Second}, nil
 			}
+
 			// Add the image puller finalizer
 			if !HasImagePullerFinalizer(ctx.CheCluster) {
 				if err := ReconcileImagePullerFinalizer(ctx); err != nil {
 					return reconcile.Result{}, err
 				}
-				return reconcile.Result{Requeue: true}, nil
+				return reconcile.Result{RequeueAfter: time.Second}, nil
 			}
 		}
 
@@ -116,7 +110,7 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 
 					// If the image puller spec is empty, set default values, update the CheCluster CR and requeue
 					// These assignments are needed because the image puller operator updates the CR with a default configmap and deployment name
-					// if none are given.  Without these, che-operator will be stuck in an update loop
+					// if none are given.  Without these, codeready-operator will be stuck in an update loop
 					if ctx.CheCluster.IsImagePullerSpecEmpty() {
 						logrus.Infof("Updating CheCluster to set KubernetesImagePuller default values")
 						_, err := UpdateImagePullerSpecIfEmpty(ctx)
@@ -124,7 +118,14 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 							logrus.Errorf("Error updating CheCluster: %v", err)
 							return reconcile.Result{}, err
 						}
-						return reconcile.Result{Requeue: true}, nil
+						return reconcile.Result{RequeueAfter: time.Second}, nil
+					}
+
+					if ctx.CheCluster.IsImagePullerImagesEmpty() {
+						if err = SetDefaultImages(ctx); err != nil {
+							logrus.Error(err)
+							return reconcile.Result{}, err
+						}
 					}
 
 					logrus.Infof("Creating KubernetesImagePuller for CheCluster %v", ctx.CheCluster.Name)
@@ -141,6 +142,11 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 				return reconcile.Result{}, err
 			}
 
+			if err = UpdateDefaultImagesIfNeeded(ctx); err != nil {
+				logrus.Error(err)
+				return reconcile.Result{}, err
+			}
+
 			if ctx.CheCluster.Spec.ImagePuller.Spec.DeploymentName == "" {
 				ctx.CheCluster.Spec.ImagePuller.Spec.DeploymentName = imagePuller.Spec.DeploymentName
 			}
@@ -156,7 +162,7 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 					logrus.Errorf("Error updating KubernetesImagePuller: %v", err)
 					return reconcile.Result{}, err
 				}
-				return reconcile.Result{Requeue: true}, nil
+				return reconcile.Result{RequeueAfter: time.Second}, nil
 			}
 		} else {
 			logrus.Infof("Waiting 15 seconds for kubernetesimagepullers.che.eclipse.org API")
@@ -172,7 +178,7 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 			}
 
 			if removed {
-				return reconcile.Result{Requeue: true}, nil
+				return reconcile.Result{RequeueAfter: time.Second}, nil
 			}
 
 			if HasImagePullerFinalizer(ctx.CheCluster) {
@@ -181,7 +187,7 @@ func ReconcileImagePuller(ctx *DeployContext) (reconcile.Result, error) {
 					logrus.Errorf("Error deleting finalizer: %v", err)
 					return reconcile.Result{}, err
 				}
-				return reconcile.Result{Requeue: true}, nil
+				return reconcile.Result{RequeueAfter: time.Second}, nil
 			}
 		}
 	}
@@ -360,14 +366,13 @@ func GetExpectedSubscription(ctx *DeployContext, packageManifest *packagesv1.Pac
 	}
 }
 
-func CompareExpectedSubscription(ctx *DeployContext, packageManifest *packagesv1.PackageManifest) (bool, error) {
-	expected := GetExpectedSubscription(ctx, packageManifest)
+func GetActualSubscription(ctx *DeployContext) (*operatorsv1alpha1.Subscription, error) {
 	actual := &operatorsv1alpha1.Subscription{}
 	err := ctx.ClusterAPI.NonCachedClient.Get(context.TODO(), types.NamespacedName{Namespace: ctx.CheCluster.Namespace, Name: "kubernetes-imagepuller-operator"}, actual)
 	if err != nil {
-		return false, err
+		return nil, err
 	}
-	return SubscriptionsAreEqual(expected, actual), nil
+	return actual, nil
 }
 
 // Update the CheCluster ImagePuller spec if the default values are not set
@@ -386,6 +391,111 @@ func UpdateImagePullerSpecIfEmpty(ctx *DeployContext) (orgv1.CheClusterSpecImage
 	return ctx.CheCluster.Spec.ImagePuller, nil
 }
 
+func SetDefaultImages(ctx *DeployContext) error {
+	defaultImages := GetDefaultImages()
+	if len(defaultImages) == 0 {
+		return nil
+	}
+	return SetImages(ctx, defaultImages)
+}
+
+// ImageSliceToString returns a string representation of the provided image slice, suitable for the
+// imagePuller.spec.images field
+func ImageSliceToString(imageSlice []ImageAndName) string {
+	var err error
+	imagesString := ""
+	for _, image := range imageSlice {
+		image.name, err = ConvertToRFC1123(image.name)
+		if err != nil {
+			continue
+		}
+		imagesString += image.name + "=" + image.image + ";"
+	}
+	return imagesString
+}
+
+// StringToImageSlice returns a slice of ImageAndName structs from the provided semi-colon seperated string
+// of key value pairs
+func StringToImageSlice(imagesString string) []ImageAndName {
+	currentImages := strings.Split(imagesString, ";")
+	for i, image := range currentImages {
+		currentImages[i] = strings.TrimSpace(image)
+	}
+	// Remove the last element, if empty
+	if currentImages[len(currentImages)-1] == "" {
+		currentImages = currentImages[:len(currentImages)-1]
+	}
+
+	images := []ImageAndName{}
+	for _, image := range currentImages {
+		nameAndImage := strings.Split(image, "=")
+		if len(nameAndImage) != 2 {
+			logrus.Warnf("Malformed image name/tag: %s. Ignoring.", image)
+			continue
+		}
+		images = append(images, ImageAndName{name: nameAndImage[0], image: nameAndImage[1]})
+	}
+
+	return images
+}
+
+// GetDefaultImages returns the current default images from the environment variables
+func GetDefaultImages() []ImageAndName {
+	images := []ImageAndName{}
+	imagePatterns := [...]string{
+		"^RELATED_IMAGE_.*_plugin_java8$",
+		"^RELATED_IMAGE_.*_plugin_java11$",
+		"^RELATED_IMAGE_.*_plugin_kubernetes$",
+		"^RELATED_IMAGE_.*_plugin_openshift$",
+		"^RELATED_IMAGE_.*_plugin_broker.*",
+		"^RELATED_IMAGE_.*_theia.*",
+		"^RELATED_IMAGE_.*_stacks_cpp$",
+		"^RELATED_IMAGE_.*_stacks_dotnet$",
+		"^RELATED_IMAGE_.*_stacks_golang$",
+		"^RELATED_IMAGE_.*_stacks_php$",
+		"^RELATED_IMAGE_.*_cpp_.*_devfile_registry_image.*",
+		"^RELATED_IMAGE_.*_dotnet_.*_devfile_registry_image.*",
+		"^RELATED_IMAGE_.*_golang_.*_devfile_registry_image.*",
+		"^RELATED_IMAGE_.*_php_.*_devfile_registry_image.*",
+		"^RELATED_IMAGE_.*_java.*_maven_devfile_registry_image.*",
+	}
+	for _, pattern := range imagePatterns {
+		matches := util.GetEnvByRegExp(pattern)
+		for _, match := range matches {
+			match.Name = match.Name[len("RELATED_IMAGE_"):]
+			images = append(images, ImageAndName{name: match.Name, image: match.Value})
+		}
+	}
+	return images
+}
+
+// Convert input string to RFC 1123 format ([a-z0-9]([-a-z0-9]*[a-z0-9])?) max 63 characters, if possible
+func ConvertToRFC1123(str string) (string, error) {
+	result := strings.ToLower(str)
+	if len(str) > validation.DNS1123LabelMaxLength {
+		result = result[:validation.DNS1123LabelMaxLength]
+	}
+
+	// Remove illegal trailing characters
+	i := len(result) - 1
+	for i >= 0 && !IsRFC1123Char(result[i]) {
+		i -= 1
+	}
+	result = result[:i+1]
+
+	result = strings.ReplaceAll(result, "_", "-")
+
+	if errs := validation.IsDNS1123Label(result); len(errs) > 0 {
+		return "", goerror.New("Cannot convert the following string to RFC 1123 format: " + str)
+	}
+	return result, nil
+}
+
+func IsRFC1123Char(ch byte) bool {
+	errs := validation.IsDNS1123Label(string(ch))
+	return len(errs) == 0
+}
+
 func CreateKubernetesImagePuller(ctx *DeployContext) (bool, error) {
 	imagePuller := GetExpectedKubernetesImagePuller(ctx)
 	err := ctx.ClusterAPI.Client.Create(context.TODO(), imagePuller, &client.CreateOptions{})
@@ -413,7 +523,46 @@ func GetExpectedKubernetesImagePuller(ctx *DeployContext) *chev1alpha1.Kubernete
 	}
 }
 
-// Unisntall the CSV, OperatorGroup, Subscription, KubernetesImagePuller, and update the CheCluster to remove
+// UpdateDefaultImagesIfNeeded updates the default images from `spec.images` if needed
+func UpdateDefaultImagesIfNeeded(ctx *DeployContext) error {
+	specImages := StringToImageSlice(ctx.CheCluster.Spec.ImagePuller.Spec.Images)
+	defaultImages := GetDefaultImages()
+	if UpdateSpecImages(specImages, defaultImages) {
+		return SetImages(ctx, specImages)
+	}
+	return nil
+}
+
+// UpdateSpecImages returns true if the default images from `spec.images` were updated
+// with new defaults
+//
+// specImages contains the images in `spec.images`
+// defaultImages contains the current default images from the environment variables
+func UpdateSpecImages(specImages []ImageAndName, defaultImages []ImageAndName) bool {
+	match := false
+	for i, specImage := range specImages {
+		specImageName, specImageTag := util.GetImageNameAndTag(specImage.image)
+		for _, defaultImage := range defaultImages {
+			defaultImageName, defaultImageTag := util.GetImageNameAndTag(defaultImage.image)
+			// if the image tags are different for this image, then update
+			if defaultImageName == specImageName && defaultImageTag != specImageTag {
+				match = true
+				specImages[i].image = defaultImage.image
+				break
+			}
+		}
+	}
+	return match
+}
+
+// SetImages sets the provided images to the imagePuller spec
+func SetImages(ctx *DeployContext, images []ImageAndName) error {
+	imagesStr := ImageSliceToString(images)
+	ctx.CheCluster.Spec.ImagePuller.Spec.Images = imagesStr
+	return UpdateCheCRSpec(ctx, "Kubernetes Image Puller images", imagesStr)
+}
+
+// Uninstall the CSV, OperatorGroup, Subscription, KubernetesImagePuller, and update the CheCluster to remove
 // the image puller spec.  Returns true if the CheCluster was updated
 func UninstallImagePullerOperator(ctx *DeployContext) (bool, error) {
 	updated := false
