@@ -13,10 +13,11 @@ package util
 
 import (
 	"bytes"
+	"context"
 	"fmt"
 	"io"
 
-	v1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	v1 "github.com/eclipse-che/che-operator/api/v1"
 	"github.com/sirupsen/logrus"
 	authorizationv1 "k8s.io/api/authorization/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -74,15 +75,22 @@ func (cl *k8s) ExecIntoPod(
 }
 
 func (cl *k8s) DoExecIntoPod(namespace string, podName string, command string, reason string) (string, error) {
+	var stdin io.Reader
+	return cl.DoExecIntoPodWithStdin(namespace, podName, command, stdin, reason)
+}
+
+func (cl *k8s) DoExecIntoPodWithStdin(namespace string, podName string, command string, stdin io.Reader, reason string) (string, error) {
 	if reason != "" {
 		logrus.Infof("Running exec for '%s' in the pod '%s'", reason, podName)
 	}
 
 	args := []string{"/bin/bash", "-c", command}
-	stdout, stderr, err := cl.RunExec(args, podName, namespace)
+	stdout, stderr, err := cl.RunExec(args, podName, namespace, stdin)
 	if err != nil {
 		logrus.Errorf("Error running exec: %v, command: %s", err, args)
-		logrus.Errorf("Stderr: %s", stderr)
+		if stderr != "" {
+			logrus.Errorf("Stderr: %s", stderr)
+		}
 		return stdout, err
 	}
 
@@ -98,7 +106,7 @@ func (cl *k8s) GetDeploymentPod(name string, ns string) (podName string, err err
 	listOptions := metav1.ListOptions{
 		LabelSelector: "component=" + name,
 	}
-	podList, _ := api.Pods(ns).List(listOptions)
+	podList, _ := api.Pods(ns).List(context.TODO(), listOptions)
 	podListItems := podList.Items
 	if len(podListItems) == 0 {
 		logrus.Errorf("Failed to find pod for component %s. List of pods: %v", name, podListItems)
@@ -116,7 +124,7 @@ func (cl *k8s) GetPodsByComponent(name string, ns string) []string {
 	listOptions := metav1.ListOptions{
 		LabelSelector: "component=" + name,
 	}
-	podList, _ := api.Pods(ns).List(listOptions)
+	podList, _ := api.Pods(ns).List(context.TODO(), listOptions)
 	for _, pod := range podList.Items {
 		names = append(names, pod.Name)
 	}
@@ -126,14 +134,14 @@ func (cl *k8s) GetPodsByComponent(name string, ns string) []string {
 
 // Reads 'user' and 'password' from the given secret
 func (cl *k8s) ReadSecret(name string, ns string) (user string, password string, err error) {
-	secret, err := cl.clientset.CoreV1().Secrets(ns).Get(name, metav1.GetOptions{})
+	secret, err := cl.clientset.CoreV1().Secrets(ns).Get(context.TODO(), name, metav1.GetOptions{})
 	if err != nil {
 		return "", "", err
 	}
 	return string(secret.Data["user"]), string(secret.Data["password"]), nil
 }
 
-func (cl *k8s) RunExec(command []string, podName, namespace string) (string, string, error) {
+func (cl *k8s) RunExec(command []string, podName, namespace string, stdin io.Reader) (string, string, error) {
 
 	req := cl.clientset.CoreV1().RESTClient().Post().
 		Resource("pods").
@@ -143,7 +151,7 @@ func (cl *k8s) RunExec(command []string, podName, namespace string) (string, str
 
 	req.VersionedParams(&corev1.PodExecOptions{
 		Command: command,
-		Stdin:   false,
+		Stdin:   stdin != nil,
 		Stdout:  true,
 		Stderr:  true,
 		TTY:     false,
@@ -156,18 +164,14 @@ func (cl *k8s) RunExec(command []string, podName, namespace string) (string, str
 	}
 
 	var stdout, stderr bytes.Buffer
-	var stdin io.Reader
 	err = exec.Stream(remotecommand.StreamOptions{
 		Stdin:  stdin,
 		Stdout: &stdout,
 		Stderr: &stderr,
 		Tty:    false,
 	})
-	if err != nil {
-		return stdout.String(), stderr.String(), err
-	}
 
-	return stdout.String(), stderr.String(), nil
+	return stdout.String(), stderr.String(), err
 }
 
 func (cl *k8s) IsResourceOperationPermitted(resourceAttr *authorizationv1.ResourceAttributes) (ok bool, err error) {
@@ -177,7 +181,7 @@ func (cl *k8s) IsResourceOperationPermitted(resourceAttr *authorizationv1.Resour
 		},
 	}
 
-	ssar, err := cl.clientset.AuthorizationV1().SelfSubjectAccessReviews().Create(lsar)
+	ssar, err := cl.clientset.AuthorizationV1().SelfSubjectAccessReviews().Create(context.TODO(), lsar, metav1.CreateOptions{})
 	if err != nil {
 		return false, err
 	}
