@@ -13,6 +13,7 @@ package util
 
 import (
 	"bytes"
+	"context"
 	"crypto/sha256"
 	"crypto/tls"
 	"encoding/base64"
@@ -30,14 +31,18 @@ import (
 	"strings"
 	"time"
 
-	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
+	orgv1 "github.com/eclipse-che/che-operator/api/v1"
+	routev1 "github.com/openshift/api/route/v1"
 	"github.com/sirupsen/logrus"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
+	k8sErrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/apimachinery/pkg/util/intstr"
 	"k8s.io/client-go/discovery"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/config"
 	"sigs.k8s.io/yaml"
 )
@@ -131,7 +136,7 @@ func getDiscoveryClient() (*discovery.DiscoveryClient, error) {
 	return discovery.NewDiscoveryClientForConfig(kubeconfig)
 }
 
-func getApiList() ([]v1.APIGroup, error) {
+func getApiList() ([]metav1.APIGroup, error) {
 	discoveryClient, err := getDiscoveryClient()
 	if err != nil {
 		return nil, err
@@ -194,17 +199,21 @@ func MergeMaps(first map[string]string, second map[string]string) map[string]str
 	return ret
 }
 
-func GetServerExposureStrategy(c *orgv1.CheCluster) string {
-	strategy := c.Spec.Server.ServerExposureStrategy
-	if strategy != "" {
-		return strategy
-	} else if c.Spec.DevWorkspace.Enable {
-		return "single-host"
-	} else if IsOpenShift {
-		return "multi-host"
-	} else {
-		return GetValue(c.Spec.K8s.IngressStrategy, "multi-host")
+func GetServerExposureStrategy(cheCluster *orgv1.CheCluster) string {
+	if cheCluster.Spec.Server.ServerExposureStrategy != "" {
+		return cheCluster.Spec.Server.ServerExposureStrategy
 	}
+
+	if !IsOpenShift && cheCluster.Spec.K8s.IngressStrategy != "" {
+		return cheCluster.Spec.K8s.IngressStrategy
+	}
+
+	// Explicitly switch to `single-host` mode
+	if cheCluster.Spec.DevWorkspace.Enable {
+		return "single-host"
+	}
+
+	return "multi-host"
 }
 
 func IsTestMode() (isTesting bool) {
@@ -219,7 +228,7 @@ func GetOpenShiftAPIUrls() (string, string, error) {
 	// for debug purpose
 	apiUrl := os.Getenv("CLUSTER_API_URL")
 	apiInternalUrl := os.Getenv("CLUSTER_API_INTERNAL_URL")
-	if apiUrl != "" && apiInternalUrl != "" {
+	if apiUrl != "" || apiInternalUrl != "" {
 		return apiUrl, apiInternalUrl, nil
 	}
 
@@ -312,6 +321,75 @@ func getAPIUrlsForOpenShiftV4() (apiUrl string, apiInternalUrl string, err error
 	return apiUrl, apiInternalUrl, nil
 }
 
+func GetRouterCanonicalHostname(client client.Client, namespace string) (string, error) {
+	testRouteYaml, err := GetTestRouteYaml(client, namespace)
+	if err != nil {
+		return "", err
+	}
+	return testRouteYaml.Status.Ingress[0].RouterCanonicalHostname, nil
+}
+
+// GetTestRouteYaml creates test route and returns its spec.
+func GetTestRouteYaml(client client.Client, namespace string) (*routev1.Route, error) {
+	// Create test route to get the info
+	routeSpec := &routev1.Route{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "Route",
+			APIVersion: routev1.SchemeGroupVersion.String(),
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      "canonical-hostname-route-name",
+			Namespace: namespace,
+		},
+		Spec: routev1.RouteSpec{
+			To: routev1.RouteTargetReference{
+				Kind: "Service",
+				Name: "canonical-hostname-route-nonexisting-service",
+			},
+			Port: &routev1.RoutePort{
+				TargetPort: intstr.IntOrString{
+					Type:   intstr.Int,
+					IntVal: 8080,
+				},
+			},
+		},
+	}
+
+	if err := client.Create(context.TODO(), routeSpec); err != nil {
+		if !k8sErrors.IsAlreadyExists(err) {
+			return nil, err
+		}
+	}
+
+	// Schedule test route cleanup after the job done.
+	defer func() {
+		if err := client.Delete(context.TODO(), routeSpec); err != nil {
+			logrus.Errorf("Failed to delete test route %s: %s", routeSpec.Name, err)
+		}
+	}()
+
+	// Wait till the route is ready
+	route := &routev1.Route{}
+	errCount := 0
+	for {
+		time.Sleep(time.Duration(1) * time.Second)
+
+		routeNsName := types.NamespacedName{Name: routeSpec.Name, Namespace: namespace}
+		err := client.Get(context.TODO(), routeNsName, route)
+		if err == nil {
+			return route, nil
+		} else if !k8sErrors.IsNotFound(err) {
+			errCount++
+			if errCount > 10 {
+				return nil, err
+			}
+		} else {
+			// Reset counter as got not found error
+			errCount = 0
+		}
+	}
+}
+
 func GetDeploymentEnv(deployment *appsv1.Deployment, key string) (value string) {
 	env := deployment.Spec.Template.Spec.Containers[0].Env
 	for i := range env {
@@ -361,6 +439,39 @@ func GetArchitectureDependentEnv(env string) string {
 	return env
 }
 
+// GetImageNameAndTag returns the image repository and tag name from the provided image
+//
+// Referenced from https://github.com/che-incubator/chectl/blob/main/src/util.ts
+func GetImageNameAndTag(image string) (string, string) {
+	var imageName, imageTag string
+	if strings.Contains(image, "@") {
+		// Image is referenced via a digest
+		index := strings.Index(image, "@")
+		imageName = image[:index]
+		imageTag = image[index+1:]
+	} else {
+		// Image is referenced via a tag
+		lastColonIndex := strings.LastIndex(image, ":")
+		if lastColonIndex == -1 {
+			imageName = image
+			imageTag = "latest"
+		} else {
+			beforeLastColon := image[:lastColonIndex]
+			afterLastColon := image[lastColonIndex+1:]
+			if strings.Contains(afterLastColon, "/") {
+				// The colon is for registry port and not for a tag
+				imageName = image
+				imageTag = "latest"
+			} else {
+				// The colon separates image name from the tag
+				imageName = beforeLastColon
+				imageTag = afterLastColon
+			}
+		}
+	}
+	return imageName, imageTag
+}
+
 // NewBoolPointer returns `bool` pointer to value in the memory.
 // Unfortunately golang hasn't got syntax to create `bool` pointer.
 func NewBoolPointer(value bool) *bool {
@@ -368,6 +479,11 @@ func NewBoolPointer(value bool) *bool {
 	return &variable
 }
 
+func IsNativeUserModeEnabled(c *orgv1.CheCluster) bool {
+	// Native user mode is now available only on openshift
+	return IsOpenShift && c.Spec.Auth.NativeUserMode != nil && *c.Spec.Auth.NativeUserMode
+}
+
 // IsOAuthEnabled returns true when oAuth is enable for CheCluster resource, otherwise false.
 func IsOAuthEnabled(c *orgv1.CheCluster) bool {
 	return IsOpenShift && c.Spec.Auth.OpenShiftoAuth != nil && *c.Spec.Auth.OpenShiftoAuth
@@ -394,6 +510,14 @@ func GetWorkspaceNamespaceDefault(cr *orgv1.CheCluster) string {
 	return GetValue(cr.Spec.Server.WorkspaceNamespaceDefault, workspaceNamespaceDefault)
 }
 
+// GetCheServerCustomCheProperty - returns value of che-server's custom property.
+func GetCheServerCustomCheProperty(cr *orgv1.CheCluster, key string) string {
+	if cr.Spec.Server.CustomCheProperties != nil {
+		return cr.Spec.Server.CustomCheProperties[key]
+	}
+	return ""
+}
+
 // IsDeleteOAuthInitialUser - returns true when initial Openshfit oAuth user must be deleted.
 func IsDeleteOAuthInitialUser(cr *orgv1.CheCluster) bool {
 	return cr.Spec.Auth.InitialOpenShiftOAuthUser != nil && !*cr.Spec.Auth.InitialOpenShiftOAuthUser && cr.Status.OpenShiftOAuthUserCredentialsSecret != ""
@@ -436,3 +560,55 @@ func ComputeHash256(data []byte) string {
 	hasher.Write(data)
 	return base64.URLEncoding.EncodeToString(hasher.Sum(nil))
 }
+
+func ReloadCheCluster(client client.Client, cheCluster *orgv1.CheCluster) error {
+	return client.Get(
+		context.TODO(),
+		types.NamespacedName{Name: cheCluster.Name, Namespace: cheCluster.Namespace},
+		cheCluster)
+}
+
+func FindCheCRinNamespace(client client.Client, namespace string) (*orgv1.CheCluster, int, error) {
+	cheClusters := &orgv1.CheClusterList{}
+	if err := client.List(context.TODO(), cheClusters); err != nil {
+		return nil, 0, err
+	}
+
+	if len(cheClusters.Items) != 1 {
+		return nil, len(cheClusters.Items), fmt.Errorf("expected an instance of CheCluster, but got %d instances", len(cheClusters.Items))
+	}
+
+	cheCR := &orgv1.CheCluster{}
+	namespacedName := types.NamespacedName{Namespace: namespace, Name: cheClusters.Items[0].GetName()}
+	err := client.Get(context.TODO(), namespacedName, cheCR)
+	if err != nil {
+		return nil, 0, err
+	}
+	return cheCR, 1, nil
+}
+
+func UpdateBackupServerConfiguration(client client.Client, backupServerConfig *orgv1.CheBackupServerConfiguration) error {
+	err := client.Update(context.TODO(), backupServerConfig)
+	if err != nil {
+		logrus.Errorf("Failed to update %s CR: %s", backupServerConfig.Name, err.Error())
+		return err
+	}
+	return nil
+}
+
+func UpdateBackupServerConfigurationStatus(client client.Client, backupServerConfig *orgv1.CheBackupServerConfiguration) error {
+	err := client.Status().Update(context.TODO(), backupServerConfig)
+	if err != nil {
+		logrus.Errorf("Failed to update %s CR status: %s", backupServerConfig.Name, err.Error())
+		return err
+	}
+	return nil
+}
+
+// ClearMetadata removes extra fields from given metadata.
+// It is required to remove ResourceVersion in order to be able to apply the yaml again.
+func ClearMetadata(objectMeta *metav1.ObjectMeta) {
+	objectMeta.ResourceVersion = ""
+
+	objectMeta.ManagedFields = []metav1.ManagedFieldsEntry{}
+}
