@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2012-2019 Red Hat, Inc.
+// Copyright (c) 2012-2021 Red Hat, Inc.
 // This program and the accompanying materials are made
 // available under the terms of the Eclipse Public License 2.0
 // which is available at https://www.eclipse.org/legal/epl-2.0/
@@ -14,12 +14,6 @@ package util
 import (
 	"reflect"
 	"testing"
-
-	"k8s.io/client-go/kubernetes/scheme"
-
-	orgv1 "github.com/eclipse-che/che-operator/pkg/apis/org/v1"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 )
 
 func TestGeneratePasswd(t *testing.T) {
@@ -52,32 +46,20 @@ func TestGetValue(t *testing.T) {
 	}
 }
 
-func TestReload(t *testing.T) {
-	cheCluster := &orgv1.CheCluster{
-		ObjectMeta: metav1.ObjectMeta{
-			Namespace:       "eclipse-che",
-			Name:            "eclipse-che",
-			ResourceVersion: "1",
-		},
+func TestGetImageNameAndTag(t *testing.T) {
+	var tests = []struct {
+		input        string
+		expectedName string
+		expectedTag  string
+	}{
+		{"quay.io/test/test:tag", "quay.io/test/test", "tag"},
+		{"quay.io/test/test@sha256:abcdef", "quay.io/test/test", "sha256:abcdef"},
+		{"quay.io/test/test", "quay.io/test/test", "latest"},
+		{"localhost:5000/test", "localhost:5000/test", "latest"},
 	}
-
-	orgv1.SchemeBuilder.AddToScheme(scheme.Scheme)
-	cli := fake.NewFakeClientWithScheme(scheme.Scheme, cheCluster)
-
-	cheCluster = &orgv1.CheCluster{
-		ObjectMeta: metav1.ObjectMeta{
-			Namespace:       "eclipse-che",
-			Name:            "eclipse-che",
-			ResourceVersion: "2",
-		},
-	}
-
-	err := ReloadCheCluster(cli, cheCluster)
-	if err != nil {
-		t.Errorf("Failed to reload checluster, %v", err)
-	}
-
-	if cheCluster.ObjectMeta.ResourceVersion != "1" {
-		t.Errorf("Failed to reload checluster")
+	for _, test := range tests {
+		if actualName, actualTag := GetImageNameAndTag(test.input); actualName != test.expectedName || actualTag != test.expectedTag {
+			t.Errorf("Test Failed. Expected '%s' and '%s', but got '%s' and '%s'", test.expectedName, test.expectedTag, actualName, actualTag)
+		}
 	}
 }
